/**
 * This file/module contains all configuration for the build process.
 */
module.exports = {
  /**
   * The `build_dir` folder is where our projects are compiled during
   * development and the `compile_dir` folder is where our app resides once it's
   * completely built.
   */
  destination_dir: 'dist',

  /**
   * This is a collection of file patterns that refer to our app code (the
   * stuff in `src/`). These file paths are used in the configuration of
   * build tasks. `js` is all project javascript, less tests. `ctpl` contains
   * our reusable components' (`src/common`) template HTML files, while
   * `atpl` contains the same, but for our app's code. `html` is just our
   * main HTML file, `less` is our main stylesheet, and `unit` contains our
   * app's unit tests.
   */
  app_files: {
    js: ['src/widget.js',
      'src/colour-lovers.js',
      'src/color-pusher.js'
    ],
    html: [ 'src/index.html' ],
    css: 'src/*.css'
  },

  /**
   * This is the same as `app_files`, except it contains patterns that
   * reference vendor code (`vendor/`) that we need to place into the build
   * process somewhere. While the `app_files` property ensures all
   * standardized files are collected for compilation, it is the user's job
   * to ensure non-standardized (i.e. vendor-related) files are handled
   * appropriately in `vendor_files.js`.
   *
   * The `vendor_files.js` property holds files to be automatically
   * concatenated and minified with our project source files.
   *
   * The `vendor_files.css` property holds any CSS files to be automatically
   * included in our app.
   */
  vendor_files: {
    js: [
      'libs/jquery.xcolor.js',
      'libs/check-types.min.js',
      'libs/check-types.color.js',
      'libs/pusher.color.min.js',
      'bower_components/jquery-minicolors/jquery.minicolors.js',
      'bower_components/angular-minicolors/angular-minicolors.js',
      'libs/ui-bootstrap-tpls-0.7.0.min.js',
      'bower_components/alertify/alertify.min.js'
    ],
    css: [
      'bower_components/jquery-minicolors/jquery.minicolors.css',
      'bower_components/alertify/themes/alertify.core.css',
      'bower_components/alertify/themes/alertify.bootstrap.css'
    ],
    assets: [
    ],
    map: [
    ]
  }
};
